﻿
using System;
using System.IO;
using Wisej.Web;

namespace Test_BackgroundTreeViewFill
{
    public partial class Page1 : Page
    {
        public delegate void onFeedbackReceivedEventHandler(object sender, string e);
        public event onFeedbackReceivedEventHandler onFinished;

        private string DocRoot = String.Empty;
        private string HTTPRoot = String.Empty;

        private enum NodeType
        {
            File,
            Directory
        };

        public Page1()
        {
            InitializeComponent();
        }

        private void Page1_Load(object sender, EventArgs e)
        {
            try
            {
                this.onFinished += BackgroundCompleted_Handler;

                DocRoot = @"C:\temp";
                Init_TreeView();
            }
            catch (Exception)
            {
                MessageBox.Show("");
            }

        }

        /// <summary>
        /// Initialize the TreeView (i.e: First load, logon Logoff, ...)
        /// </summary>
        private void Init_TreeView()
        {
            try
            {
                treeView1.SuspendLayout();
                treeView1.Nodes.Clear();
                BuildTree(new DirectoryInfo(DocRoot), treeView1.Nodes);
                treeView1.Sort();
                //Expand the first node
                treeView1.Nodes[0].Expand();
                treeView1.SelectedNode = treeView1.Nodes[0];
                treeView1.ResumeLayout();
            }
            catch (Exception ex)
            {
                AlertBox.Show("Error in 'Init_TreeView'" + Environment.NewLine + ex.Message, MessageBoxIcon.Error);
            }

        }
        private void BuildTree(DirectoryInfo directoryInfo, TreeNodeCollection addInMe)
        {
            try
            {
                bool HiddenDir = false;

                //Hidden directories are not to be displayed for NOT logged on users
                if ((directoryInfo.Attributes & FileAttributes.Hidden) != 0)
                {
                    if (Application.Session["isAuthenticated"] == false)
                    {
                        return;
                    }
                    else
                    {
                        HiddenDir = true;
                    }
                }

                TreeNode curNode = addInMe.Add(directoryInfo.FullName, directoryInfo.Name);
                curNode.Tag = NodeType.Directory;
                try
                {
                    foreach (FileInfo file in directoryInfo.GetFiles())
                    {
                        string FExtension = System.IO.Path.GetExtension(file.Name).ToUpper();

                        TreeNode FileTN = new TreeNode(); FileTN.Name = file.FullName; FileTN.Text = file.Name; FileTN.Tag = NodeType.File;
                        if (HiddenDir) FileTN.BackColor = System.Drawing.Color.LightGray;

                        curNode.Nodes.Add(FileTN);
                    }

                    foreach (DirectoryInfo subdir in directoryInfo.GetDirectories())
                    {
                        BuildTree(subdir, curNode.Nodes);
                    }
                }
                catch (Exception ex)
                {
                    AlertBox.Show("Error occured in 'BuildTree' : " + ex.Message);
                }
            }
            catch (Exception ex)
            {
                AlertBox.Show("Error in 'BuildTree'" + Environment.NewLine + ex.Message, MessageBoxIcon.Error);
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            System.Threading.Tasks.Task.Factory.StartNew(() =>
        {
            try
            {
                onFinished(this, "Background Text");

            }
            catch (Exception ex)
            {
                AlertBox.Show("Error occured " + Environment.NewLine + ex.Message);
            }
        });
        }

        private void BackgroundCompleted_Handler(object sender, string e)
        {
            try
            {
                MessageBox.Show("Background Process is Finished");

                Init_TreeView();

                //Application.StartTask(() => { Init_TreeView(); });
            }
            catch (Exception)
            {
                MessageBox.Show("Error");
            }
        }
    }
}
